// 2023/01/29 DEBUG

#include <ArduinoBLE.h>     // version 1.3.1

#define SW         A1       // for re-start SCAN
#define LOOPTIME  100       // Loop timer mS
#define DATANUM     8       // Sample payload millis(), micros() 8bytes
union unionData {           // union for bit conversion
  uint32_t  data32[DATANUM/4];
  uint16_t  data16[DATANUM/2];  
  uint8_t   data8[DATANUM];
};
union unionData ud;
bool readingFlag = false;   // Data reading flag
int err = 127;              // scan_connect error code
int ss;                     // RSSI

//"myPeripheral"
//"a4e21aa1-f8ea-45bc-bf5c-07e6cecf6e7f" 
//"1B:61:BF:A0:75:42"
#define myUUID(val) ("a4e21aa1-" val "-45bc-bf5c-07e6cecf6e7f")
BLECharacteristic dataCharacteristic(myUUID("0010"), BLEWritten, DATANUM);

BLEDevice peripheral;

void setup() 
{
  Serial.begin(1115200);
//  while (!Serial);
  delay(5000);

  pinMode(LED_RED, OUTPUT);       // LED
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);
  pinMode(SW, INPUT_PULLUP);      // ON:LOW OFF:HOGH

  digitalWrite(LED_RED, HIGH);    // LED OFF
  digitalWrite(LED_GREEN, HIGH);
  digitalWrite(LED_BLUE, HIGH);

  BLE.begin();
}

//**************************************************************************************************
void loop() 
{
  while(digitalRead(SW) == HIGH) // while SW=OFF, BLE.end()
  {
    Serial.println("SW OFF");
    BLE.end();
    delay(500);
  }
  BLE.begin();                // when SW=ON, BLE.begin()
  do{
    digitalWrite(LED_BLUE, LOW);
    BLE.poll();
    err = scan_connect();     // scan connect function
    Serial.print("ERROR : ");
    Serial.println(err);
    digitalWrite(LED_BLUE, HIGH);
  }while(err != 0);
  
    digitalWrite(LED_GREEN, LOW);
      
    while (peripheral.connected()) 
    {
      unsigned long timestamp = millis();
 
      readingFlag = true;
        uint32_t mS = ud.data32[0];
        uint32_t uS = ud.data32[1];
      readingFlag = false;
              
        Serial.print(mS); Serial.print(" ");
        Serial.print(uS); Serial.print(" ");
        Serial.println(ss);
    
      while(millis() - timestamp <= LOOPTIME);   // Print data every LOOPTIME mS 
    }
  
  digitalWrite(LED_GREEN, HIGH);
}

//**************************************************************************************************
int scan_connect(void)
{
  // 1.
  Serial.println("1.SCANNING ................"); 
  if(!BLE.scanForUuid(myUUID("0000")))
  {
    Serial.println("1x.Scan error");      
  }
  Serial.println("1.Scanned");
  
  // 2.
  peripheral = BLE.available();
  if (!peripheral)
  {
    Serial.println("2x.Peripheral unabailable");
    return 2;
  }
//  Serial.println("2.Peripheral abailable");

  // 3.
  if (peripheral.localName() != "myPeripheral")
  {
    Serial.println("3x.Peripheral local name miss match");
    return 3;
  }
//  Serial.println("3.Got the right peripheral");

  // 4.
  BLE.stopScan();
  Serial.println("4.Stop scanning");

  // 5.
  Serial.println("5.CONNECTING ................");
  if (!peripheral.connect()) 
  {
    Serial.println("5x.Can't connect");
    return 5;
  } 
//  Serial.println("5.Connected");

  // 6.
//  Serial.println("6.Discover attributes");  
  if (!peripheral.discoverService(myUUID("0000")))
  {
    Serial.println("6x.Didn't discover attributes");
    peripheral.disconnect();
    return 6;
  }
//  Serial.println("6.discovered attributes");

  // 7.
  dataCharacteristic = peripheral.characteristic(myUUID("0010"));
  dataCharacteristic.setEventHandler(BLEWritten, characteristicWritten);  //BLEWritten event handler
//  Serial.println("7.Char and eventhandler setting");  

  // 8.
//  Serial.println("8.can subscribe?");
  if (!dataCharacteristic.canSubscribe())
  {
    Serial.println("8x.Can't subscribe");
    peripheral.disconnect();
    return 8;
  }
//  Serial.println("8.Can subscribe");

  // 9.
//  Serial.println("9.Scribe");
  if (!dataCharacteristic.subscribe())
  {
    Serial.println("9x.Can't Subscribe");
    peripheral.disconnect();
    return 9;
  }
//  Serial.println("9.Subscribed");

  // 10.
  Serial.println("10.Success scanning and connecting");
  return 0;
}

//***************************************************************************************
//characteristic written event handler
void characteristicWritten(BLEDevice peripheral, BLECharacteristic thisChar) 
{
    digitalWrite(LED_RED, LOW);
    
  while(readingFlag == true)  {       // Data update prohibited
  }       
  dataCharacteristic.readValue(ud.data8, DATANUM);  
  int s = peripheral.rssi();
  if ((s != 0) & (s != 127)) {        // Meaningless 0,127
    ss = s;
  }
  
    long timestamp = millis();
    while(millis() - timestamp <= 1);  // LED ON timer
    
    digitalWrite(LED_RED, HIGH);
}
