// 2023/01/29 DEBUG

#include <ArduinoBLE.h>     // version 1.3.1

#define LOOPTIME  500       // Loop timer mS
#define DATANUM     8       // Sample payload millis(), micros() 8bytes
union unionData {           // union for bit conversion
  uint32_t  data32[DATANUM/4];
  uint16_t  data16[DATANUM/2];
  uint8_t   data8[DATANUM];
};
union unionData ud;

// "a4e21aa1-xxxx-45bc-bf5c-07e6cecf6e7f"
// "1B:61:BF:A0:75:42"
#define myUUID(val) ("a4e21aa1-" val "-45bc-bf5c-07e6cecf6e7f")
BLEService        data1Service(myUUID("0000"));
BLECharacteristic data1Characteristic(myUUID("0010"), BLENotify, DATANUM);

void setup() {
  Serial.begin(115200);
//  while (!Serial);
  delay(5000);

  pinMode(LED_RED, OUTPUT);       // LED
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);

  digitalWrite(LED_RED, HIGH);    // LED OFF
  digitalWrite(LED_GREEN, HIGH);
  digitalWrite(LED_BLUE, HIGH);

  if (!BLE.begin())
  {
    Serial.println("starting BLE module failed!");
    while (1);
  }

  BLE.setLocalName("myPeripheral");
  BLE.setAdvertisedService(data1Service);
  data1Service.addCharacteristic(data1Characteristic);
  BLE.addService(data1Service); 
  BLE.advertise();

  Serial.println("myPeripheral is running");
}

void loop() 
{
  BLEDevice central = BLE.central();
  
  if (central)
  {
    digitalWrite(LED_GREEN, LOW);
    Serial.println("Got a central");
    while (central.connected())
    {    
      digitalWrite(LED_RED, LOW);
      ud.data32[0] = millis();    // Sample payload
      ud.data32[1] = micros();
      data1Characteristic.writeValue(ud.data8, DATANUM);
                    
      for(int i=0; i <DATANUM; i++){
        Serial.print(ud.data8[i]); Serial.print(" ");                       
      }
      Serial.println();
                    
      delay(5);
      digitalWrite(LED_RED, HIGH);

      unsigned long timestamp = millis();
      while(millis() - timestamp <= LOOPTIME);
    }
    digitalWrite(LED_GREEN, HIGH);
    Serial.print("Disconnected from central: ");
    Serial.println(central.address());
  }
  delay(100);
}
